@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,8 @@
 
 package com.openexchange.authentication.ldap;
 
+import java.util.HashMap;
+import java.util.Map;
 import java.util.Properties;
 import javax.naming.AuthenticationException;
 import javax.naming.Context;
@@ -66,6 +68,8 @@ import com.openexchange.authentication.Authenticated;
 import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
+import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Reloadable;
 import com.openexchange.exception.OXException;
 import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
 
@@ -73,7 +77,10 @@ import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
  * This class implements the login by using an LDAP for authentication.
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
-public class LDAPAuthentication implements AuthenticationService {
+public class LDAPAuthentication implements AuthenticationService, Reloadable {
+
+    private static final String CONFIGFILE = "ldapauth.properties";
+    private static final String[] PROPERTIES = new String[] {"all properties in file"};
 
     private static final class AuthenticatedImpl implements Authenticated {
 
@@ -121,7 +128,7 @@ public class LDAPAuthentication implements AuthenticationService {
     /**
      * Properties for the JNDI context.
      */
-    private final Properties props;
+    private Properties props;
 
     /**
      * attribute name and base DN.
@@ -208,18 +215,23 @@ public class LDAPAuthentication implements AuthenticationService {
                 cons.setSearchScope(SearchControls.SUBTREE_SCOPE);
                 cons.setCountLimit(0);
                 cons.setReturningAttributes(new String[]{"dn"});
-                NamingEnumeration<SearchResult> res = context.search(baseDN, filter, cons);
-                if( res.hasMoreElements() ) {
-                    dn = res.nextElement().getNameInNamespace();
+                NamingEnumeration<SearchResult> res = null;
+                try {
+                    res = context.search(baseDN, filter, cons);
                     if( res.hasMoreElements() ) {
-                        final String errortext = "Found more then one user with " + uidAttribute + "=" + uid;
+                        dn = res.nextElement().getNameInNamespace();
+                        if( res.hasMoreElements() ) {
+                            final String errortext = "Found more then one user with " + uidAttribute + "=" + uid;
+                            LOG.error(errortext);
+                            throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
+                        }
+                    } else {
+                        final String errortext = "No user found with " + uidAttribute + "=" + uid;
                         LOG.error(errortext);
-                        throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
+                        throw LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING.create(uid);
                     }
-                } else {
-                    final String errortext = "No user found with " + uidAttribute + "=" + uid;
-                    LOG.error(errortext);
-                    throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
+                } finally {
+                    close(res);
                 }
                 context.close();
             } else {
@@ -251,27 +263,32 @@ public class LDAPAuthentication implements AuthenticationService {
                     searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE);
                     searchControls.setCountLimit(0);
                     searchControls.setReturningAttributes(new String[]{ldapReturnField});
-                    final NamingEnumeration<SearchResult> search;
+                    NamingEnumeration<SearchResult> search = null;
                     NamingEnumeration<SearchResult> searchProxy = null;
-                    if (null == samAccountName) {
-                        if( proxyAs != null ) {
-                            search = context.search(this.baseDN, "(displayName=" + uid + ")", searchControls);
-                            searchProxy = context.search(this.baseDN, "(displayName=" + proxyAs + ")", searchControls);
+                    try {
+                        if (null == samAccountName) {
+                            if( proxyAs != null ) {
+                                search = context.search(this.baseDN, "(displayName=" + uid + ")", searchControls);
+                                searchProxy = context.search(this.baseDN, "(displayName=" + proxyAs + ")", searchControls);
+                            } else {
+                                search = context.search(this.baseDN, "(displayName=" + uid + ")", searchControls);
+                            }
                         } else {
-                            search = context.search(this.baseDN, "(displayName=" + uid + ")", searchControls);
+                            search = context.search(this.baseDN, "(sAMAccountName=" + samAccountName + ")", searchControls);
                         }
-                    } else {
-                        search = context.search(this.baseDN, "(sAMAccountName=" + samAccountName + ")", searchControls);
-                    }
-                    if (null != search && search.hasMoreElements()) {
-                        final SearchResult next = search.next();
-                        userDnAttributes = next.getAttributes();
-                        if( proxyAs != null && searchProxy != null ) {
-                            puser = (String)searchProxy.next().getAttributes().get(ldapReturnField).get();
+                        if (null != search && search.hasMoreElements()) {
+                            final SearchResult next = search.next();
+                            userDnAttributes = next.getAttributes();
+                            if( proxyAs != null && searchProxy != null ) {
+                                puser = (String)searchProxy.next().getAttributes().get(ldapReturnField).get();
+                            }
+                        } else {
+                            LOG.error("No user with displayname {} found.", uid);
+                            throw LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING.create(uid);
                         }
-                    } else {
-                        LOG.error("No user with displayname {} found.", uid);
-                        throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
+                    } finally {
+                        close(search);
+                        close(searchProxy);
                     }
                 } else {
                     userDnAttributes = context.getAttributes(dn);
@@ -389,4 +406,38 @@ public class LDAPAuthentication implements AuthenticationService {
         }
         return splitted;
     }
+
+    @Override
+    public void reloadConfiguration(ConfigurationService configService) {
+        Properties properties = configService.getFile(CONFIGFILE);
+        this.props = properties;
+        try {
+            init();
+        } catch (OXException e) {
+            LOG.error("Error reloading configuration for bundle com.openexchange.authentication.ldap: {}", e);
+        }
+    }
+
+    @Override
+    public Map<String, String[]> getConfigFileNames() {
+        Map<String, String[]> map = new HashMap<String, String[]>(1);
+        map.put(CONFIGFILE, PROPERTIES);
+        return map;
+    }
+
+    /**
+     * Closes the supplied naming enumeration, swallowing a possible {@link NamingException}.
+     *
+     * @param namingEnumeration The naming operation to close, or <code>null</code> to do nothing for convenience
+     */
+    private static void close(NamingEnumeration<?> namingEnumeration) {
+        if (null != namingEnumeration) {
+            try {
+                namingEnumeration.close();
+            } catch (NamingException e) {
+                LOG.warn("Error closing naming enumeration", e);
+            }
+        }
+    }
+
 }
